Stan Math Library: stan/math/prim/scal/prob/inv_chi_square_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

inv_chi_square_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_INV_CHI_SQUARE_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_INV_CHI_SQUARE_LOG_HPP
3
4 #include <boost/random/chi_squared_distribution.hpp>
5 #include <boost/random/variate_generator.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_nonnegative.hpp>
9 #include <stan/math/prim/scal/err/check_not_nan.hpp>
10 #include <stan/math/prim/scal/err/check_positive_finite.hpp>
11 #include <stan/math/prim/scal/fun/constants.hpp>
12 #include <stan/math/prim/scal/fun/multiply_log.hpp>
13 #include <stan/math/prim/scal/fun/value_of.hpp>
14 #include <stan/math/prim/scal/fun/gamma_q.hpp>
15 #include <stan/math/prim/scal/fun/digamma.hpp>
16 #include <stan/math/prim/scal/meta/length.hpp>
17 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
18 #include <stan/math/prim/scal/meta/VectorView.hpp>
19 #include <stan/math/prim/scal/meta/VectorBuilder.hpp>
20 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
21 #include <stan/math/prim/scal/meta/return_type.hpp>
22 #include <stan/math/prim/scal/fun/grad_reg_inc_gamma.hpp>
23 #include <stan/math/prim/scal/meta/include_summand.hpp>
24 #include <cmath>
25
26 namespace stan {
27
28 namespace math {
29
49 template <bool propto,
50 typename T_y, typename T_dof>
51 typename return_type<T_y, T_dof>::type
52 inv_chi_square_log(const T_y& y, const T_dof& nu) {
53 static const char* function("stan::math::inv_chi_square_log");
54 typedef typename stan::partials_return_type<T_y, T_dof>::type
55 T_partials_return;
56
57 // check if any vectors are zero length
58 if (!(stan::length(y)
59 && stan::length(nu)))
60 return 0.0;
61
62 using stan::math::check_positive_finite;
63 using stan::math::check_not_nan;
64 using stan::math::value_of;
65 using stan::math::check_consistent_sizes;
66
67 T_partials_return logp(0.0);
68 check_positive_finite(function, "Degrees of freedom parameter", nu);
69 check_not_nan(function, "Random variable", y);
70 check_consistent_sizes(function,
71 "Random variable", y,
72 "Degrees of freedom parameter", nu);
73
74
75 // set up template expressions wrapping scalars into vector views
76 VectorView<const T_y> y_vec(y);
77 VectorView<const T_dof> nu_vec(nu);
78 size_t N = max_size(y, nu);
79
80 for (size_t n = 0; n < length(y); n++)
81 if (value_of(y_vec[n]) <= 0)
82 return LOG_ZERO;
83
84 using boost::math::digamma;
85 using boost::math::lgamma;
86 using stan::math::multiply_log;
87 using std::log;
88
89 VectorBuilder<include_summand<propto, T_y, T_dof>::value,
90 T_partials_return, T_y> log_y(length(y));
91 for (size_t i = 0; i < length(y); i++)
92 if (include_summand<propto, T_y, T_dof>::value)
93 log_y[i] = log(value_of(y_vec[i]));
94
95 VectorBuilder<include_summand<propto, T_y>::value,
96 T_partials_return, T_y> inv_y(length(y));
97 for (size_t i = 0; i < length(y); i++)
98 if (include_summand<propto, T_y>::value)
99 inv_y[i] = 1.0 / value_of(y_vec[i]);
100
101 VectorBuilder<include_summand<propto, T_dof>::value,
102 T_partials_return, T_dof> lgamma_half_nu(length(nu));
103 VectorBuilder<!is_constant_struct<T_dof>::value,
104 T_partials_return, T_dof>
105 digamma_half_nu_over_two(length(nu));
106 for (size_t i = 0; i < length(nu); i++) {
107 T_partials_return half_nu = 0.5 * value_of(nu_vec[i]);
108 if (include_summand<propto, T_dof>::value)
109 lgamma_half_nu[i] = lgamma(half_nu);
110 if (!is_constant_struct<T_dof>::value)
111 digamma_half_nu_over_two[i] = digamma(half_nu) * 0.5;
112 }
113
114 OperandsAndPartials<T_y, T_dof> operands_and_partials(y, nu);
115 for (size_t n = 0; n < N; n++) {
116 const T_partials_return nu_dbl = value_of(nu_vec[n]);
117 const T_partials_return half_nu = 0.5 * nu_dbl;
118
119 if (include_summand<propto, T_dof>::value)
120 logp += nu_dbl * NEG_LOG_TWO_OVER_TWO - lgamma_half_nu[n];
121 if (include_summand<propto, T_y, T_dof>::value)
122 logp -= (half_nu+1.0) * log_y[n];
123 if (include_summand<propto, T_y>::value)
124 logp -= 0.5 * inv_y[n];
125
126 if (!is_constant_struct<T_y>::value) {
127 operands_and_partials.d_x1[n]
128 += -(half_nu+1.0) * inv_y[n] + 0.5 * inv_y[n] * inv_y[n];
129 }
130 if (!is_constant_struct<T_dof>::value) {
131 operands_and_partials.d_x2[n]
132 += NEG_LOG_TWO_OVER_TWO - digamma_half_nu_over_two[n]
133 - 0.5*log_y[n];
134 }
135 }
136 return operands_and_partials.value(logp);
137 }
138
139 template <typename T_y, typename T_dof>
140 inline
141 typename return_type<T_y, T_dof>::type
142 inv_chi_square_log(const T_y& y, const T_dof& nu) {
143 return inv_chi_square_log<false>(y, nu);
144 }
145 }
146 }
147 #endif
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
stan::math::lgamma
fvar< T > lgamma(const fvar< T > &x)
Definition: lgamma.hpp:15
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
check_positive_finite.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
digamma.hpp
stan::math::log
fvar< T > log(const fvar< T > &x)
Definition: log.hpp:15
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
is_constant_struct.hpp
stan::math::LOG_ZERO
const double LOG_ZERO
Definition: constants.hpp:175
return_type.hpp
stan::math::include_summand
Template metaprogram to calculate whether a summand needs to be included in a proportional (log) prob...
Definition: include_summand.hpp:36
gamma_q.hpp
stan::return_type::type
boost::math::tools::promote_args< typename scalar_type< T1 >::type, typename scalar_type< T2 >::type, typename scalar_type< T3 >::type, typename scalar_type< T4 >::type, typename scalar_type< T5 >::type, typename scalar_type< T6 >::type >::type type
Definition: return_type.hpp:27
include_summand.hpp
multiply_log.hpp
check_nonnegative.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
VectorView.hpp
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
VectorBuilder.hpp
partials_return_type.hpp
OperandsAndPartials.hpp
length.hpp
stan::math::multiply_log
fvar< T > multiply_log(const fvar< T > &x1, const fvar< T > &x2)
Definition: multiply_log.hpp:15
grad_reg_inc_gamma.hpp
stan::VectorBuilder
VectorBuilder allocates type T1 values to be used as intermediate values.
Definition: VectorBuilder.hpp:28
stan::math::inv_chi_square_log
return_type< T_y, T_dof >::type inv_chi_square_log(const T_y &y, const T_dof &nu)
The log of an inverse chi-squared density for y with the specified degrees of freedom parameter...
Definition: inv_chi_square_log.hpp:52
constants.hpp
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
stan::math::NEG_LOG_TWO_OVER_TWO
const double NEG_LOG_TWO_OVER_TWO
Definition: constants.hpp:191
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
stan::partials_return_type::type
boost::math::tools::promote_args< typename partials_type< typename scalar_type< T1 >::type >::type, typename partials_type< typename scalar_type< T2 >::type >::type, typename partials_type< typename scalar_type< T3 >::type >::type, typename partials_type< typename scalar_type< T4 >::type >::type, typename partials_type< typename scalar_type< T5 >::type >::type, typename partials_type< typename scalar_type< T6 >::type >::type >::type type
Definition: partials_return_type.hpp:26
check_consistent_sizes.hpp
stan::math::check_positive_finite
bool check_positive_finite(const char *function, const char *name, const T_y &y)
Return true if y is positive and finite.
Definition: check_positive_finite.hpp:28
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39
stan::math::digamma
fvar< T > digamma(const fvar< T > &x)
Definition: digamma.hpp:16

[ Stan Home Page ] © 2011–2016, Stan Development Team.